const helpers = require('./helpers');

const CommonsChunkPlugin = require('webpack/lib/optimize/CommonsChunkPlugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const HtmlElementsPlugin = require('./html-elements-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const InlineManifestWebpackPlugin = require('inline-manifest-webpack-plugin');
const LoaderOptionsPlugin = require('webpack/lib/LoaderOptionsPlugin');
const ScriptExtHtmlWebpackPlugin = require('script-ext-html-webpack-plugin');
const ngcWebpack = require('ngc-webpack');
const buildUtils = require('./build-utils');

/*
 * Webpack 常量配置
 */
module.exports = function (options) {
	const isProd = options.env === 'product';
	const METADATA = Object.assign({}, buildUtils.DEFAULT_METADATA, options.metadata || {});
	const ngcWebpackConfig = buildUtils.ngcWebpackSetup(isProd, METADATA);
	
	const entry = {
		polyfills: './src/polyfills.browser.ts',
		main: './src/main.browser.ts'
	};
	Object.assign(ngcWebpackConfig.plugin, {
		tsConfigPath: METADATA.tsConfigPath,
		mainPath: entry.main
	});

	return {
		entry: entry,
		resolve: {
			mainFields: ['browser', 'module', 'main'],
			extensions: ['.ts', '.js', '.json'],
			modules: [helpers.root('src'), helpers.root('node_modules')],
		},
		module: {

			rules: [
				...ngcWebpackConfig.loaders,
				{
					test: /\.css$/,
					use: ['to-string-loader', 'css-loader'],
					exclude: [helpers.root('src', 'styles')]
				},
				{
					test: /\.scss$/,
					use: ['to-string-loader', 'css-loader', 'sass-loader'],
					exclude: [helpers.root('src', 'styles')]
				},
				{
					test: /\.html$/,
					use: 'raw-loader',
					exclude: [helpers.root('src/index.html')]
				},
				{
					test: /\.(jpg|png|gif)$/,
					use: 'file-loader'
				},

				{
					test: /\.(eot|woff2?|svg|ttf)([\?]?.*)$/,
					use: 'file-loader'
				}
			],
		},
		plugins: [
			new CommonsChunkPlugin({
				name: 'polyfills',
				chunks: ['polyfills']
			}),
			new CommonsChunkPlugin({
				minChunks: Infinity,
				name: 'inline'
			}),
			new CommonsChunkPlugin({
				name: 'main',
				async: 'common',
				children: true,
				minChunks: 2
			}),
			new CopyWebpackPlugin([{
						from: 'src/assets',
						to: 'assets'
					},
					{
						from: 'src/meta'
					}
				],
				isProd ? {
					ignore: ['mock-data/**/*']
				} : undefined
			),
			new HtmlWebpackPlugin({
				template: 'src/index.html',
				title: METADATA.title,
				chunksSortMode: function (a, b) {
					const entryPoints = ["inline", "polyfills", "sw-register", "styles", "vendor", "main"];
					return entryPoints.indexOf(a.names[0]) - entryPoints.indexOf(b.names[0]);
				},
				metadata: METADATA,
				inject: 'body',
				xhtml: true,
				minify: isProd ? {
					caseSensitive: true,
					collapseWhitespace: true,
					keepClosingSlash: true
				} : false
			}),
			new ScriptExtHtmlWebpackPlugin({
				sync: /inline|polyfills|vendor/,
				defaultAttribute: 'async',
				preload: [/polyfills|vendor|main/],
				prefetch: [/chunk/]
			}),
			new HtmlElementsPlugin({
				headTags: require('./head-config.common')
			}),

			new LoaderOptionsPlugin({}),

			new ngcWebpack.NgcWebpackPlugin(ngcWebpackConfig.plugin),
			new InlineManifestWebpackPlugin(),
		],
		node: {
			global: true,
			crypto: 'empty',
			process: true,
			module: false,
			clearImmediate: false,
			setImmediate: false
		}

	};
}